package com.ekexiu.console.service;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.concurrent.atomic.AtomicReference;

import org.jfw.util.exception.JfwBaseException;
import org.jfw.util.io.IoUtil;
import org.jfw.util.web.fileupload.Item;
import org.jfw.util.web.fileupload.UploadItemIterator;

public class Upload {
	public static final int ERROR_NO_FOUND_SUFFIX = JfwBaseException.MIN_APP_ERROR_CODE + 1;
	public static final int ERROR_INVALID_SUFFIX = JfwBaseException.MIN_APP_ERROR_CODE + 2;
	public static final int ERROR_INVALID_SIZE = JfwBaseException.MIN_APP_ERROR_CODE + 3;
	// public static final int ERROR_NO_FOUND_SUFFIX =
	// JfwBaseException.MIN_APP_ERROR_CODE+1;

	private File path;
	private String validSuffix;
	private boolean includeSuffix;
	private long maxSize = Long.MAX_VALUE;
	private long rate = 24 * 60 * 60 * 1000;
	private int buffSize = 8092;
	private AtomicReference<String> lastPathWithCreate = new AtomicReference<String>(null);

	public int getBuffSize() {
		return buffSize;
	}

	public void setBuffSize(int buffSize) {
		this.buffSize = buffSize;
	}

	public long getRate() {
		return rate;
	}

	public void setRate(long rate) {
		this.rate = rate;
	}

	public File getPath() {
		return path;
	}

	public void setPath(File path) {
		this.path = path;
	}

	public String getValidSuffix() {
		return validSuffix;
	}

	public void setValidSuffix(String validSuffix) {
		this.validSuffix = validSuffix;
	}

	public boolean isIncludeSuffix() {
		return includeSuffix;
	}

	public void setIncludeSuffix(boolean includeSuffix) {
		this.includeSuffix = includeSuffix;
	}

	public long getMaxSize() {
		return maxSize;
	}

	public void setMaxSize(long maxSize) {
		this.maxSize = maxSize;
	}

	public void setUploadPath(File path) {
		this.path = path;
	}

	private void checkSuffix(String ext) throws JfwBaseException {
		if (ext == null) {
			if (this.includeSuffix) {
				throw new JfwBaseException(ERROR_NO_FOUND_SUFFIX, "upload file no found suffix");
			}
		} else {
			if (null != this.validSuffix && this.validSuffix.indexOf("," + ext + ",") < 0) {
				throw new JfwBaseException(ERROR_INVALID_SUFFIX, "upload file suffix  invalid");
			}
		}
	}

	public UploadItem upload(UploadItemIterator it) throws IOException, JfwBaseException {
		while (it.hasNext()) {
			Item item = it.next();
			if (!item.isFormField()) {
				String fn = item.getName();
				String ext = null;
				int index = fn.lastIndexOf('.');
				if (index >= 0) {
					ext = fn.substring(index + 1);
					if (ext.length() == 0) {
						ext = null;
					}
				}
				this.checkSuffix(ext);
				UploadItem ui = new UploadItem();
				ui.setName(fn);
				String uri = this.findValidUri(ext);
				File dest = new File(this.path, uri);
				while (dest.exists()) {
					uri = this.findValidUri(ext);
					dest = new File(this.path, uri);
				}
				FileOutputStream os = new FileOutputStream(dest);
				try {
					byte[] buf = new byte[this.buffSize];
					InputStream in = item.getInputStream();
					long size = 0;
					try {
						int len = 0;
						while ((len = in.read(buf)) >= 0) {
							if (len > 0) {
								os.write(buf, 0, len);
								size += len;
								if (size > this.maxSize) {
									throw new JfwBaseException(ERROR_INVALID_SIZE, "upload file size is larger");
								}
							}
						}
						os.flush();
						ui.setSize(size);
						ui.setUri(uri);
						return ui;
					} finally {
						IoUtil.close(in);
					}
				} finally {
					IoUtil.close(os);
				}
			}
		}
		return null;
	}

	private String findValidUri(String ext) throws IOException {
		StringBuilder sb = new StringBuilder();
		sb.append(Long.toHexString(System.currentTimeMillis() / this.rate));
		String p = sb.toString();
		if (!p.equals(lastPathWithCreate.get())) {
			File cp = new File(this.path, p);
			if (!cp.exists()) {
				if (!cp.mkdirs()) {
					throw new IOException("create path[" + cp.getAbsolutePath() + "] error");
				}
			} else if (cp.isFile()) {
				throw new IOException("create path[" + cp.getAbsolutePath() + "] error,exists same name file");
			}
			lastPathWithCreate.set(p);
		}
		sb.append("/").append(Long.toHexString(System.nanoTime()));
		if (ext != null)
			sb.append(".").append(ext);
		return sb.toString();
	}

	public static class UploadItem {
		private String name;
		private String uri;
		private long size;

		public String getName() {
			return name;
		}

		public void setName(String name) {
			this.name = name;
		}

		public String getUri() {
			return uri;
		}

		public void setUri(String uri) {
			this.uri = uri;
		}

		public long getSize() {
			return size;
		}

		public void setSize(long size) {
			this.size = size;
		}

	}
}
