package com.ekexiu.console.system.dao;

import com.ekexiu.console.system.po.User;
import org.jfw.apt.annotation.Nullable;
import org.jfw.apt.orm.annotation.dao.Batch;
import org.jfw.apt.orm.annotation.dao.DAO;
import org.jfw.apt.orm.annotation.dao.Dynamic;
import org.jfw.apt.orm.annotation.dao.method.*;
import org.jfw.apt.orm.annotation.dao.method.operator.*;
import org.jfw.apt.orm.annotation.dao.param.*;
import org.jfw.util.PageQueryResult;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.List;

@DAO
public interface UserDao {

	@Insert
	int insert(Connection con, User user) throws SQLException;

	@Nullable
	@SelectOne
	User queryById(Connection con, String id) throws SQLException;

	@Or
	@Nullable
	@SelectOne
	User queryByLoginKey(Connection con, @Alias({ "mobile", "email" }) String key) throws SQLException;

	@Update
	@Dynamic
	int update(Connection con, User user) throws SQLException;

	@Batch
	@IncludeFixSet("modifyTime")
	@UpdateWith
	@From(User.class)
	int[] resetPassword(Connection con,@Batch  String[] id, @Set String loginPassword) throws SQLException;

	@UpdateWith
	@From(User.class)
	int changePassword(Connection con, String id,@Set String loginPassword)throws SQLException;

	@IncludeFixSet("modifyTime")
	@UpdateWith
	@From(User.class)
	int update(Connection con, String id, @Set String authinfo) throws SQLException;

	@Where("ACTIVED<>'1'")
	@SetSentence("ACTIVED='1'")
	@IncludeFixSet("modifyTime")
	@From(User.class)
	@UpdateWith
	int enable(Connection con, String id) throws SQLException;

	@Where("ACTIVED='1'")
	@SetSentence("ACTIVED='0'")
	@IncludeFixSet("modifyTime")
	@From(User.class)
	@UpdateWith
	int dsiable(Connection con, String id) throws SQLException;

	@PageSelect
	PageQueryResult<User> pageQuery(Connection con, @Nullable @Like String name, @Nullable Boolean actived, @Nullable String mobile,
			@Nullable String email, @Nullable @GtEq @Alias("createTime") String bt, @Nullable @Alias("createTime") @LtEq String et, int pageNo, int pageSize)
			throws SQLException;

	@SelectList
	List<User> queryall(Connection con) throws SQLException;

	@DeleteWith
	@From(User.class)
	int delete(Connection con, String id) throws SQLException;

	@DeleteWith
	@Batch
	@From(User.class)
	int[] delete(Connection con, @Batch String[] id) throws SQLException;

	@Batch
	@UpdateWith
	@From(User.class)
	int[] delete(Connection con, @Batch String[] id, @Set boolean actived) throws SQLException;
}
