package com.ekexiu.console.system.service;

import com.ekexiu.console.system.dao.DictDao;
import com.ekexiu.console.system.po.Dict;
import com.ekexiu.console.system.vo.DictInfo;
import com.ekexiu.console.system.vo.DictItem;
import com.ekexiu.console.system.vo.EditDictItem;
import org.jfw.apt.annotation.Autowrie;
import org.jfw.apt.annotation.DefaultValue;
import org.jfw.apt.annotation.Nullable;
import org.jfw.apt.web.annotation.Path;
import org.jfw.apt.web.annotation.operate.Get;
import org.jfw.apt.web.annotation.operate.Post;
import org.jfw.apt.web.annotation.operate.Put;
import org.jfw.apt.web.annotation.param.JdbcConn;
import org.jfw.apt.web.annotation.param.PathVar;
import org.jfw.apt.web.annotation.param.RequestBody;
import org.jfw.apt.web.annotation.param.RequestParam;
import org.jfw.util.PageQueryResult;
import org.jfw.util.exception.JfwBaseException;
import org.jfw.util.jdbc.JdbcTask;
import org.jfw.util.jdbc.JdbcUtil;
import org.jfw.util.json.JsonService;
import org.jfw.util.reflect.TypeReference;

import javax.sql.DataSource;
import java.lang.reflect.Type;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.*;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicReference;

@Path("/sys/dict")
public class DictService {

	private final static Type LINKEDLIST_EDITDICTITEM_TYPE = new TypeReference<List<EditDictItem>>() {
	}.getType();

	private AtomicReference<Map<String, DictInfo>> dicts = new AtomicReference<Map<String, DictInfo>>();
	private Map<String, AtomicReference<LinkedList<DictItem>>> items = new HashMap<String, AtomicReference<LinkedList<DictItem>>>();

	@Autowrie
	private DictDao dictDao;

	public DictDao getDictDao() {
		return dictDao;
	}

	public void setDictDao(DictDao dictDao) {
		this.dictDao = dictDao;
	}

	@Get
	@Path("/all")
	public List<DictInfo> getAll() {
		Collection<DictInfo> ds = this.dicts.get().values();
		ArrayList<DictInfo> ret = new ArrayList<DictInfo>(ds.size());
		ret.addAll(ds);
		return ret;
	}

	private ArrayList<DictInfo> filterAndSort(String key) {
		ArrayList<DictInfo> ret = new ArrayList<DictInfo>();
		Collection<DictInfo> its = this.dicts.get().values();
		if (key == null || key.length() == 0) {
			ret.addAll(its);
		} else {
			for (DictInfo di : its) {
				if (di.getCode().indexOf(key) >= 0 || di.getCaption().indexOf(key) >= 0) {
					ret.add(di);
				}
			}
		}
		Collections.sort(ret, new Comparator<DictInfo>(){
			@Override
			public int compare(DictInfo o1, DictInfo o2) {
				return o1.getCode().compareTo(o2.getCode());
			}
		});
		return ret;
	}
	
	@Get
	@Path("/pq")
	public PageQueryResult<DictInfo> pageQuery( @Nullable String key,@DefaultValue("1") int pageNo,@DefaultValue("10") int pageSize){
		 ArrayList<DictInfo> list = filterAndSort(key);
		return PageQueryResult.build(list, pageSize, pageNo);
	}

	@Get
	@Path("/info/{code}")
	public DictInfo get(@PathVar String code) {
		return this.dicts.get().get(code);
	}

	@Get
	@Path("/item/{code}")
	public List<DictItem> getItems(@PathVar String code) {
		AtomicReference<LinkedList<DictItem>> ref = this.items.get(code);
		return ref != null ? ref.get() : Collections.<DictItem> emptyList();
	}

	@Get
	@Path("/editItem/{code}")
	public List<EditDictItem> getItems(@JdbcConn Connection con, @PathVar String code) throws SQLException {
		DictInfo di = this.dicts.get().get(code);
		if (di == null || di.isReadonly())
			return Collections.<EditDictItem> emptyList();

		String val = this.dictDao.queryItem(con, code);
		if (val == null || val.length() < 3)
			return Collections.<EditDictItem> emptyList();
		return JsonService.fromJson(val, LINKEDLIST_EDITDICTITEM_TYPE);
	}

	@Put
	@Path("/info")
	public void update(@JdbcConn final Connection con, @RequestBody final Dict dict) throws SQLException {
		final AtomicBoolean ab = new AtomicBoolean(false);
		JdbcUtil.executeAutoCommit(con, new JdbcTask() {

			@Override
			public void exec() throws SQLException {
				ab.set(dictDao.update(con, dict) > 0);
			}
		});

		if (ab.get())
			this.update(dict);
	}
	@Post
	@Path("/info")
	public void insert(@JdbcConn final Connection con, @RequestParam(excludeFields="item") final Dict dict)throws SQLException{
		final AtomicBoolean ab = new AtomicBoolean(false);
		JdbcUtil.executeAutoCommit(con, new JdbcTask() {
			@Override
			public void exec() throws SQLException {
				ab.set(dictDao.insert(con, dict) > 0);
			}
		});
		if (ab.get())
			this.update(dict);
	}

	@Put
	@Path("/item/{code}")
	public void update(@JdbcConn(true) final Connection con, @PathVar final String code, @RequestBody List<EditDictItem> items)
			throws SQLException, JfwBaseException {
		DictInfo di = this.dicts.get().get(code);
		if (di == null) {
			throw new JfwBaseException(50001, "invalid parameter code: not exists dict code");
		}
		if (di.isReadonly()) {
			throw new JfwBaseException(50002, "invalid parameter code: readonly dict code");
		}
		// TODO : check dictItem.system == true can't modify
		LinkedList<DictItem> ret = new LinkedList<DictItem>();
		this.transfer(items,ret);
		final AtomicBoolean ab = new AtomicBoolean(false);
		final String val = JsonService.toJson(items);
		JdbcUtil.executeAutoCommit(con, new JdbcTask() {
			public void exec() throws SQLException {
				if (dictDao.update(con, code, val) > 0)
					ab.set(true);
			}
		});
		if (ab.get()) {
			AtomicReference<LinkedList<DictItem>>  ref = this.items.get(code);
			if(ref ==null){
				ref = new AtomicReference<LinkedList<DictItem>>();
				ref.set(ret);
				this.items.put(code,ref);

			}else{
				ref.set(ret);
			}
		}
	}

	// public static List
	public DictItem queryItem(String dictCode, String itemCode) {
		AtomicReference<LinkedList<DictItem>> aitems = this.items.get(dictCode);
		if (null != aitems) {
			LinkedList<DictItem> ims = aitems.get();
			if (ims != null && ims.size() > 0) {
				return query(ims, itemCode);
			}
		}
		return null;
	}

	private DictItem query(LinkedList<DictItem> items, String code) {
		for (ListIterator<DictItem> it = items.listIterator(); it.hasNext();) {
			DictItem item = it.next();
			if (item.getCode().equals(code))
				return item;
			LinkedList<DictItem> citems = item.getChildren();
			if (citems.size() > 0) {
				item = query(citems, code);
				if (null != item)
					return item;
			}
		}
		return null;
	}

	public LinkedList<DictItem> transfer(List<EditDictItem> src, LinkedList<DictItem> dest) {
		for(ListIterator<EditDictItem> it = src.listIterator();it.hasNext();){
			EditDictItem edi = it.next();
			DictItem id = transfer(edi);
			dest.add(id);
			LinkedList<EditDictItem> child = edi.getChildren();
			if(null!= child && child.size()>0){
				LinkedList<DictItem> ldi =new LinkedList<DictItem>();
				id .setChildren(ldi);
				transfer(child,ldi);
			}
		}
		return dest;
	}

	public DictItem transfer(EditDictItem eitem) {
		DictItem item = new DictItem();
		item.setCaption(eitem.getCaption());
		item.setCode(eitem.getCode());
		item.setData(eitem.getData());
		item.setEnabled(eitem.isEnabled());
		item.setShortCode(eitem.getShortCode());
		item.setSystem(eitem.isSystem());
		return item;
	}

	private LinkedList<DictItem> transfer(String s) {
		LinkedList<DictItem> ret = new LinkedList<DictItem>();
		if (s == null || s.length() == 0 || s.equals("[]")) {
			return ret;
		}
		try {
			List<EditDictItem> list = JsonService.fromJson(s, LINKEDLIST_EDITDICTITEM_TYPE);
			if (null == transfer(list, ret)) {
				// TODO:: // log not exists parentCode or loop parentCode

			}
		} catch (Exception e) {
			// TODO: ///// log json parse exception

		}
		return ret;
	}

	public void initByDataSource(DataSource ds) throws SQLException {
		final Connection con = ds.getConnection();
		JdbcUtil.executeAutoClose(con, new JdbcTask() {
			@Override
			public void exec() throws SQLException {
				List<Dict> list = DictService.this.dictDao.queryAlll(con);
				DictService.this.init(list);
			}
		}, false);
	}

	private synchronized void init(List<Dict> list) {
		if (null != this.dicts.get())
			return;
		Map<String, DictInfo> map = new HashMap<String, DictInfo>();
		for (Dict dict : list) {
			map.put(dict.getCode(), new DictInfo(dict));
			AtomicReference<LinkedList<DictItem>> ar = new AtomicReference<LinkedList<DictItem>>(transfer(dict.getItem()));
			this.items.put(dict.getCode(), ar);
		}
		this.dicts.set(map);
	}

	private void update(Dict dict) {
		Map<String, DictInfo> map = new HashMap<String, DictInfo>();
		map.putAll(this.dicts.get());
		map.remove(dict.getCode());
		map.put(dict.getCode(), new DictInfo(dict));
		this.dicts.set(map);
	}

}
