package com.ekexiu.console.system.service;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicReference;

import javax.sql.DataSource;

import org.jfw.apt.annotation.Autowrie;
import org.jfw.apt.web.annotation.Path;
import org.jfw.apt.web.annotation.operate.Delete;
import org.jfw.apt.web.annotation.operate.Get;
import org.jfw.apt.web.annotation.operate.Post;
import org.jfw.apt.web.annotation.operate.Put;
import org.jfw.apt.web.annotation.param.JdbcConn;
import org.jfw.apt.web.annotation.param.PathVar;
import org.jfw.apt.web.annotation.param.RequestBody;
import org.jfw.util.jdbc.JdbcTask;
import org.jfw.util.jdbc.JdbcUtil;

import com.ekexiu.console.system.dao.RightDao;
import com.ekexiu.console.system.po.SysRight;

@Path("/sys/right")
public class RightService {
	private AtomicReference<Map<Integer, SysRight>> rights = new AtomicReference<Map<Integer, SysRight>>();

	@Autowrie
	private RightDao rightDao;

	public RightDao getRightDao() {
		return rightDao;
	}

	public void setRightDao(RightDao rightDao) {
		this.rightDao = rightDao;
	}

	@Get
	@Path("/all")
	public Collection<SysRight> query()  {
		return this.rights.get().values();
	}

	@Get
	@Path("/type/{type}")
	public List<SysRight> queryByType(@PathVar String type) throws SQLException {
		ArrayList<SysRight> list = new ArrayList<SysRight>();
		for (SysRight right : this.query()) {
			if (type.equals(right.getType()))
				list.add(right);
		}
		return list;
	}

	@Get
	@Path("/id/{id}")
	public SysRight queryById(@JdbcConn Connection con, @PathVar int id) throws SQLException {
		return this.rights.get().get(id);
	}

	@Post
	@Path()
	public void insert(@JdbcConn final Connection con, final SysRight right) throws SQLException {
		JdbcUtil.executeAutoCommit(con, new JdbcTask() {
			@Override
			public void exec() throws SQLException {
				rightDao.insert(con, right);
			}
		});
		this.update(right);
	}

	@Put
	@Path()
	public void update(@JdbcConn final Connection con, @RequestBody final SysRight right) throws SQLException {
		final AtomicReference<Boolean> ab = new AtomicReference<Boolean>(false);
		JdbcUtil.executeAutoCommit(con, new JdbcTask() {
			@Override
			public void exec() throws SQLException {
				ab.set(rightDao.update(con, right)>0);
			}
		});
		if(ab.get())this.update(right);
	}

	@Path("/{id}")
	@Delete
	public void delete(@JdbcConn final Connection con, @PathVar final int id) throws SQLException {
		final AtomicReference<Boolean> ab = new AtomicReference<Boolean>(false);
		JdbcUtil.executeAutoCommit(con, new JdbcTask() {
			@Override
			public void exec() throws SQLException {
				ab.set(rightDao.delete(con, id)>0);
			}
		});
		if(ab.get())this.remove(id);
	}

	public void initByDataSource(DataSource ds) throws SQLException {
		final Connection con = ds.getConnection();
		JdbcUtil.executeAutoClose(con, new JdbcTask() {
			@Override
			public void exec() throws SQLException {
				List<SysRight> list =rightDao. query(con);
				Map<Integer, SysRight> map = new HashMap<Integer, SysRight>();
				for (SysRight right : list) {
					map.put(right.getId(), right);
				}
				rights.set(map);
			}
		}, false);
	}

	private void update(SysRight right) {
		Map<Integer, SysRight> map = new HashMap<Integer, SysRight>();
		map.putAll(rights.get());
		map.remove(right.getId());
		map.put(right.getId(), right);
		rights.set(map);
	}

	private void remove(int rightId) {
		Map<Integer, SysRight> map = new HashMap<Integer, SysRight>();
		map.putAll(rights.get());
		map.remove(rightId);
		rights.set(map);
	}

}
