package com.ekexiu.console.system.service;

import com.ekexiu.console.system.dao.RoleDao;
import com.ekexiu.console.system.dao.UserRoleDao;
import com.ekexiu.console.system.po.Role;
import com.ekexiu.console.system.po.UserRole;
import com.ekexiu.console.system.vo.ConsoleAuthRole;
import org.jfw.apt.annotation.Autowrie;
import org.jfw.apt.web.annotation.Path;
import org.jfw.apt.web.annotation.operate.Get;
import org.jfw.apt.web.annotation.operate.Post;
import org.jfw.apt.web.annotation.operate.Put;
import org.jfw.apt.web.annotation.param.JdbcConn;
import org.jfw.apt.web.annotation.param.PathVar;
import org.jfw.apt.web.annotation.param.RequestBody;
import org.jfw.util.StringUtil;
import org.jfw.util.auth.AuthUtil;
import org.jfw.util.jdbc.JdbcTask;
import org.jfw.util.jdbc.JdbcUtil;

import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.*;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicReference;

@Path("/sys/role")
public class RoleService {

	public static final String EMPTY_AUTH_STR = AuthUtil.serialAuth(new int[] { 0 });
	private AtomicReference<Map<String, ConsoleAuthRole>> cacheRoles = new AtomicReference<Map<String, ConsoleAuthRole>>();

	@Autowrie
	private RoleDao roleDao;
	@Autowrie
	private UserRoleDao userRoleDao;

	public UserRoleDao getUserRoleDao() {
		return userRoleDao;
	}

	public void setUserRoleDao(UserRoleDao userRoleDao) {
		this.userRoleDao = userRoleDao;
	}

	public RoleDao getRoleDao() {
		return roleDao;
	}

	public void setRoleDao(RoleDao roleDao) {
		this.roleDao = roleDao;
	}

	@Post
	@Path
	public void insert(@JdbcConn final Connection con, ConsoleAuthRole role) throws SQLException {
		String id = StringUtil.buildUUID();
		role.setId(id);

		final Role r = new Role();
		r.setId(id);
		r.setName(role.getName());
		r.setDescp(role.getDescp());
		r.setType(role.getType());
		int[] auths = role.getAuths();
		if (auths == null || auths.length == 0) {
			r.setAuthinfo(EMPTY_AUTH_STR);
		} else {
			r.setAuthinfo(AuthUtil.serialAuth(AuthUtil.compress(auths)));
		}
		JdbcUtil.executeAutoCommit(con, new JdbcTask() {
			@Override
			public void exec() throws SQLException {

				roleDao.insert(con, r);
			}
		});
		this.update(role);
	}

	@Put
	@Path
	public void update(@JdbcConn final Connection con, @RequestBody final ConsoleAuthRole role) throws SQLException {
		final AtomicBoolean ab = new AtomicBoolean(false);
		JdbcUtil.executeAutoCommit(con, new JdbcTask() {
			@Override
			public void exec() throws SQLException {

				Role r = new Role();
				r.setId(role.getId());
				r.setType(role.getType());
				r.setName(role.getName());
				r.setDescp(role.getDescp());
				int[] auths = role.getAuths();
				if (auths == null || auths.length == 0) {
					r.setAuthinfo(EMPTY_AUTH_STR);
				} else {
					r.setAuthinfo(AuthUtil.serialAuth(AuthUtil.compress(auths)));
				}
				ab.set(roleDao.update(con, r) > 0);
			}
		});
		if (ab.get())
			this.update(role);
	}

	@Path("/{id}")
	@Post
	public int delete(@JdbcConn final Connection con, @PathVar final String id) throws SQLException {
		final AtomicBoolean ab = new AtomicBoolean(false);
		UserRole userRole = userRoleDao.queryByRoleId(con, id);
		if (userRole == null) {
			JdbcUtil.executeAutoCommit(con, new JdbcTask() {
				@Override
				public void exec() throws SQLException {
					//userRoleDao.deleteByRoleId(con, id);
					ab.set(roleDao.delete(con, id) > 0);
				}
			});
			if (ab.get())
				this.remove(id);
			return 1;
		}
		return 0;
	}

	@Path("/all")
	@Get
	public Collection<ConsoleAuthRole> queryAll() throws SQLException {
		return this.cacheRoles.get().values();
	}

	@Path("/id/{id}")
	@Get
	public ConsoleAuthRole queryById(@PathVar String id) throws SQLException {
		return this.cacheRoles.get().get(id);
	}

	@Path("/byUserid/{userid}")
	@Get
	public List<ConsoleAuthRole> queryByUserId(@JdbcConn Connection con, @PathVar String userid) throws SQLException {
		List<Role> roles = this.roleDao.queryByUser(con, userid);
		ArrayList<ConsoleAuthRole> ret = new ArrayList<ConsoleAuthRole>(roles.size());
		for (ListIterator<Role> it = roles.listIterator(); it.hasNext();) {
			ret.add(conver(it.next()));
		}
		return ret;
	}

	public static ConsoleAuthRole conver(Role role) {
		if (null == role)
			return null;
		ConsoleAuthRole cr = new ConsoleAuthRole();
		cr.setDescp(role.getDescp());
		cr.setId(role.getId());
		cr.setType(role.getType());
		cr.setName(role.getName());
		cr.setAuths(AuthUtil.unCompress(AuthUtil.deSerialAuth(role.getAuthinfo())));
		return cr;
	}

	public void initByDataSource(DataSource ds) throws SQLException {
		final Connection con = ds.getConnection();
		JdbcUtil.executeAutoClose(con, new JdbcTask() {
			@Override
			public void exec() throws SQLException {
				List<Role> list = roleDao.query(con);
				Map<String, ConsoleAuthRole> map = new HashMap<String, ConsoleAuthRole>();
				for (Role role : list) {
					ConsoleAuthRole car = conver(role);
					map.put(car.getId(), car);
				}
				cacheRoles.set(map);
			}
		}, false);
	}

	private void update(ConsoleAuthRole car) {
		Map<String, ConsoleAuthRole> map = new HashMap<String, ConsoleAuthRole>();
		map.putAll(cacheRoles.get());
		map.remove(car.getId());
		map.put(car.getId(), car);
		cacheRoles.set(map);
	}

	private void remove(String id) {
		Map<String, ConsoleAuthRole> map = new HashMap<String, ConsoleAuthRole>();
		map.putAll(cacheRoles.get());
		map.remove(id);
		cacheRoles.set(map);
	}



	public static void main(String[] args){
		String s =
		AuthUtil.serialAuth(AuthUtil.compress(new int[]{1,2,3,4,5,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32}));
		System.out.println(s);

	}
}
