package com.ekexiu.console.system.service;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicReference;

import javax.sql.DataSource;

import org.jfw.apt.annotation.Autowrie;
import org.jfw.apt.web.annotation.LoginUser;
import org.jfw.apt.web.annotation.Path;
import org.jfw.apt.web.annotation.operate.Delete;
import org.jfw.apt.web.annotation.operate.Get;
import org.jfw.apt.web.annotation.operate.Post;
import org.jfw.apt.web.annotation.operate.Put;
import org.jfw.apt.web.annotation.param.JdbcConn;
import org.jfw.apt.web.annotation.param.PathVar;
import org.jfw.apt.web.annotation.param.RequestBody;
import org.jfw.util.jdbc.JdbcTask;
import org.jfw.util.jdbc.JdbcUtil;

import com.ekexiu.console.system.dao.SysResourceDao;
import com.ekexiu.console.system.po.SysResource;
import com.ekexiu.console.system.vo.ConsoleAuthUser;

@Path("/sys/resource")
public class SysResourceService {
	private AtomicReference<Map<String, SysResource>> cache = new AtomicReference<Map<String, SysResource>>();
	@Autowrie
	private SysResourceDao sysResourceDao;

	public SysResourceDao getSysResourceDao() {
		return sysResourceDao;
	}

	public void setSysResourceDao(SysResourceDao sysResourceDao) {
		this.sysResourceDao = sysResourceDao;
	}

	@Get
	@Path("/all")
	public Collection<SysResource> query() throws SQLException {
		return this.cache.get().values();
	}

	@Post
	@Path
	public void insert(@JdbcConn(true) final Connection con, final SysResource sres) throws SQLException {
		JdbcUtil.executeAutoCommit(con, new JdbcTask() {
			@Override
			public void exec() throws SQLException {
				sysResourceDao.insert(con, sres);
			}
		});
		this.update(sres);
	}

	@Put
	@Path
	public void update(@JdbcConn(true) final Connection con,@RequestBody final SysResource sres) throws SQLException {
		final AtomicBoolean ab = new AtomicBoolean(false);
		JdbcUtil.executeAutoCommit(con, new JdbcTask() {
			@Override
			public void exec() throws SQLException {
				ab.set(sysResourceDao.update(con, sres) > 0);
			}
		});
		if (ab.get())
			this.update(sres);
	}

	@Delete
	@Path("/id/{id}")
	public void delete(@JdbcConn(true) final Connection con, @PathVar final String id) throws SQLException {
		final AtomicBoolean ab = new AtomicBoolean(false);
		JdbcUtil.executeAutoCommit(con, new JdbcTask() {
			@Override
			public void exec() throws SQLException {
				ab.set(sysResourceDao.delete(con, id) > 0);
			}
		});
		if (ab.get())
			this.remove(id);
	}

	@Get
	@Path("/user")
	public List<SysResource> query(@JdbcConn Connection con, @LoginUser ConsoleAuthUser user) throws SQLException {
		List<SysResource> ret = new ArrayList<SysResource>();
		for (SysResource res :this.cache.get().values()) {
			if (user.hasAuthority(res.getAuthRight())) {
				ret.add(res);
			}
		}
		return ret;
	}

	public void initByDataSource(DataSource ds) throws SQLException {
		final Connection con = ds.getConnection();
		JdbcUtil.executeAutoClose(con, new JdbcTask() {
			@Override
			public void exec() throws SQLException {
				List<SysResource> list = sysResourceDao.query(con);
				Map<String, SysResource> map = new HashMap<String, SysResource>();
				for (SysResource res : list) {
					map.put(res.getId(), res);
				}
				cache.set(map);
			}
		}, false);
	}

	private void update(SysResource res) {
		Map<String, SysResource> map = new HashMap<String, SysResource>();
		map.putAll(cache.get());
		map.remove(res.getId());
		map.put(res.getId(), res);
		cache.set(map);
	}

	private void remove(String id) {
		Map<String, SysResource> map = new HashMap<String, SysResource>();
		map.putAll(cache.get());
		map.remove(id);
		cache.set(map);
	}
}
