package com.ekexiu.console.system.service;

import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Type;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import org.jfw.apt.annotation.Autowrie;
import org.jfw.apt.web.annotation.LoginUser;
import org.jfw.apt.web.annotation.Path;
import org.jfw.apt.web.annotation.operate.Get;
import org.jfw.apt.web.annotation.operate.Put;
import org.jfw.apt.web.annotation.param.JdbcConn;
import org.jfw.apt.web.annotation.param.PathVar;
import org.jfw.util.ConstData;
import org.jfw.util.io.IoUtil;
import org.jfw.util.json.JsonService;
import org.jfw.util.reflect.TypeReference;

import com.ekexiu.console.system.dao.UserConfigDao;
import com.ekexiu.console.system.po.SysResource;
import com.ekexiu.console.system.po.UserConfig;
import com.ekexiu.console.system.vo.ConsoleAuthUser;
import com.ekexiu.console.system.vo.UserMenu;

@Path("/user/config")
public class UserConfigService {

	@Autowrie
	private UserConfigDao userConfigDao;
	@Autowrie
	private SysResourceService sysResourceService;

	public SysResourceService getSysResourceService() {
		return sysResourceService;
	}

	public void setSysResourceService(SysResourceService sysResourceService) {
		this.sysResourceService = sysResourceService;
	}

	public UserConfigDao getUserConfigDao() {
		return userConfigDao;
	}

	public void setUserConfigDao(UserConfigDao userConfigDao) {
		this.userConfigDao = userConfigDao;
	}

	@Put
	@Path("/{cfgkey}")
	public void insert(@LoginUser ConsoleAuthUser user, @JdbcConn(true) Connection con, @PathVar String cfgkey, InputStream in)
			throws SQLException, IOException {
		byte[] bs = IoUtil.readStream(in, true);
		this.userConfigDao.delete(con, user.getId(), cfgkey);
		if (bs.length > 0) {
			UserConfig uc = new UserConfig();
			uc.setCfkey(cfgkey);
			uc.setCfval(new String(bs, ConstData.UTF8));
			uc.setUserid(user.getId());
			this.userConfigDao.insert(con, uc);
		}
	}

	public <T> T load(Connection con, String userid, String cfkey, Type t) throws SQLException {
		UserConfig uc = this.userConfigDao.query(con, userid, cfkey);
		if (uc != null) {
			String v = uc.getCfval();
			if (v != null && v.length() > 0)
				return JsonService.fromJson(v, t);
		}
		return null;
	}

	private final static Type MENU_TYPE = new TypeReference<List<UserMenu>>() {
	}.getType();

	@Get
	@Path("/menu")
	public List<UserMenu> menu(@JdbcConn Connection con, @LoginUser() ConsoleAuthUser user) throws SQLException {
		List<UserMenu> ret = this.load(con, user.getId(), "menu", MENU_TYPE);
		if (null == ret || ret.isEmpty()) {
			ret = new ArrayList<UserMenu>();
			List<SysResource> reses = this.sysResourceService.query(con, user);
			for (SysResource res : reses) {
				if (res.isMain())
					ret.add(transfer(res));
			}
		}
		return ret;
	}

	public UserMenu transfer(SysResource res) {
		UserMenu menu = new UserMenu();
		menu.setCaption(res.getName());
		menu.setModal(false);
		menu.setRes(res.getId());
		return menu;
	}

}
