package com.ekexiu.console.util;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.StringWriter;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.net.HttpURLConnection;
import java.net.URL;

import org.jfw.util.ConstData;
import org.jfw.util.exception.JfwBaseException;
import org.jfw.util.io.IoUtil;
import org.jfw.util.json.JsonService;
import org.jfw.util.reflect.TypeReference;

public class HttpServiceImpl implements HttpService {
	private static final byte[] SIMPLE_POST_DATA = "1=1".getBytes(ConstData.UTF8);

	private String charset = "UTF-8";

	public Type buildResultType(final Type t) {
		return new ParameterizedType() {

			@Override
			public Type getRawType() {
				return Result.class;
			}

			@Override
			public Type getOwnerType() {
				return HttpServiceImpl.class;
			}

			@Override
			public Type[] getActualTypeArguments() {
				return new Type[] { t };
			}
		};
	}

	private void raise(Result<?> ret) throws JfwBaseException {
		throw new JfwBaseException(ret.getCode().intValue(), "remote calll error:" + ret.getMsg() + ";\r\n" + ret.getDetailMsg());
	}

	private <T> T handResponse(InputStream in, Type typeOfT) throws JfwBaseException {
		if (null != typeOfT) {
			Result<T> ret = JsonService.fromJson(new InputStreamReader(in, ConstData.UTF8), buildResultType(typeOfT));
			if (ret.isSuccess()) {
				return ret.getData();
			} else {
				raise(ret);
			}

		} else {
			Result<String> ret = JsonService.fromJson(new InputStreamReader(in, ConstData.UTF8), new TypeReference<Result<String>>() {
			}.getType());
			if (ret.isSuccess())
				return null;
			else {
				raise(ret);
			}
		}
		return null;
	}

	@Override
	public <T> T get(String url, Type typeOfT) throws JfwBaseException, IOException {
		URL u = new URL(url);
		HttpURLConnection httpURLConnection = (HttpURLConnection) u.openConnection();
		httpURLConnection.setRequestProperty("Accept-Charset", charset);
		httpURLConnection.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");
		if (httpURLConnection.getResponseCode() != 200) {
			throw new IOException("HTTP Request is not success, Response code is " + httpURLConnection.getResponseCode());
		}
		InputStream in = httpURLConnection.getInputStream();
		try {
			return this.handResponse(in, typeOfT);
		} finally {
			IoUtil.close(in);
		}
	}

	@Override
	public <T> T post(String url, String data, Type typeOfT) throws JfwBaseException, IOException {
		byte[] bs = SIMPLE_POST_DATA;
		if (data != null && data.length() > 0) {
			bs = data.getBytes(ConstData.UTF8);
		}
		URL u = new URL(url);
		HttpURLConnection httpURLConnection = (HttpURLConnection) u.openConnection();
		httpURLConnection.setDoOutput(true);
		httpURLConnection.setRequestMethod("POST");
		httpURLConnection.setRequestProperty("Accept-Charset", charset);
		httpURLConnection.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");
		httpURLConnection.setRequestProperty("Content-Length", String.valueOf(bs.length));
		OutputStream outputStream = httpURLConnection.getOutputStream();
		try {
			outputStream.write(bs);
			outputStream.flush();
		} finally {
			IoUtil.close(outputStream);
		}
		if (httpURLConnection.getResponseCode() != 200) {
			throw new IOException("HTTP Request is not success, Response code is " + httpURLConnection.getResponseCode());
		}
		InputStream in = httpURLConnection.getInputStream();
		try {
			return this.handResponse(in, typeOfT);
		} finally {
			IoUtil.close(in);
		}
	}

	@Override
	public <T> T put(String url, Object data, Type typeOfT) throws JfwBaseException, IOException {
		StringWriter sw = new StringWriter();

		JsonService.toJson(data, sw);
		byte[] bs = sw.toString().getBytes(ConstData.UTF8);

		URL u = new URL(url);
		HttpURLConnection httpURLConnection = (HttpURLConnection) u.openConnection();
		httpURLConnection.setDoOutput(true);
		httpURLConnection.setRequestMethod("PUT");
		httpURLConnection.setRequestProperty("Accept-Charset", charset);
		httpURLConnection.setRequestProperty("Content-Type", "application/json");
		httpURLConnection.setRequestProperty("Content-Length", String.valueOf(bs.length));
		OutputStream outputStream = httpURLConnection.getOutputStream();
		try {
			outputStream.write(bs);
			outputStream.flush();
		} finally {
			IoUtil.close(outputStream);
		}
		if (httpURLConnection.getResponseCode() != 200) {
			throw new IOException("HTTP Request is not success, Response code is " + httpURLConnection.getResponseCode());
		}
		InputStream in = httpURLConnection.getInputStream();
		try {
			return this.handResponse(in, typeOfT);
		} finally {
			IoUtil.close(in);
		}
	}

	@Override
	public <T> T delete(String url, Type typeOfT) throws JfwBaseException, IOException {
		URL u = new URL(url);
		HttpURLConnection httpURLConnection = (HttpURLConnection) u.openConnection();
		httpURLConnection.setRequestProperty("Accept-Charset", charset);
		httpURLConnection.setRequestMethod("DELETE");
		httpURLConnection.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");
		if (httpURLConnection.getResponseCode() != 200) {
			throw new IOException("HTTP Request is not success, Response code is " + httpURLConnection.getResponseCode());
		}
		InputStream in = httpURLConnection.getInputStream();
		try {
			return this.handResponse(in, typeOfT);
		} finally {
			IoUtil.close(in);
		}
	}

	public static class Result<T> {
		private boolean success;
		private Integer code;
		private String msg;
		private String detailMsg;
		private T data;

		public boolean isSuccess() {
			return success;
		}

		public void setSuccess(boolean success) {
			this.success = success;
		}

		public Integer getCode() {
			return code;
		}

		public void setCode(Integer code) {
			this.code = code;
		}

		public String getMsg() {
			return msg;
		}

		public void setMsg(String msg) {
			this.msg = msg;
		}

		public String getDetailMsg() {
			return detailMsg;
		}

		public void setDetailMsg(String detailMsg) {
			this.detailMsg = detailMsg;
		}

		public T getData() {
			return data;
		}

		public void setData(T data) {
			this.data = data;
		}
	}

}
